#pragma once
#include "Vector2.h"
#include <vector>
#include <map>
#include "Mesh.h"

enum EdgeWalkerCode {
	x, y, z, uvx, uvy
};

struct IEdgeCodeGetter {
	virtual float get(EdgeWalkerCode) = 0;
};

struct Package : public IEdgeCodeGetter{
	Vec3f vertex;
	Vec2f uv;

	Package() = default;

	Package(Vec3f vertex, Vec2f uv) :vertex(vertex), uv(uv) {

	}

	Package lerp(Package& v, float weight) {
		Package package;
		package.vertex = vertex.lerp(v.vertex, weight);
		package.uv = uv.lerp(v.uv, weight);
		return package;
	}

	float get(EdgeWalkerCode code) {
		switch (code) {
		case x:
			return vertex.x;
		case y:
			return vertex.y;
		case z:
			return vertex.z;
		case uvx:
			return uv.x;
		case uvy:
			return uv.y;
		default:
			return 0;
		}
	}


};

struct FromTo {
	EdgeWalkerCode edgeWalkerCode;
	float from, to;

	FromTo(EdgeWalkerCode edgeWalkerCode, float from, float to):edgeWalkerCode(edgeWalkerCode), from(from), to(to) {
		
	}
};

struct EdgeWalker : public IEdgeCodeGetter {
	int counter;
	std::map<EdgeWalkerCode, float> posMap;
	std::map<EdgeWalkerCode, float> incsMap;

	EdgeWalker(float from, float to, std::vector<FromTo> fromtos) {
		
		counter = (int)ceil(from - 0.5f);
		float diff = to - from;

		for each (FromTo package in fromtos){
			posMap[package.edgeWalkerCode] = package.from;
			incsMap[package.edgeWalkerCode] = (package.to - package.from) / diff;
		}

	}

	

	static EdgeWalker copyOver(IEdgeCodeGetter& a, IEdgeCodeGetter& b, EdgeWalkerCode walker, std::vector<EdgeWalkerCode> codes) {
		std::vector<FromTo> fromtos;
		for each (EdgeWalkerCode code in codes){
			fromtos.push_back(FromTo(code, a.get(code), b.get(code)));
		}

		return EdgeWalker(a.get(walker), b.get(walker), fromtos);
	}

	void step() {
		counter++;
		std::map<EdgeWalkerCode, float>::iterator it = posMap.begin();

		while (it != posMap.end()) {
			posMap[it->first] += incsMap[it->first];
			it++;
		}
	}


	float get(EdgeWalkerCode code) {
		return posMap[code];
	}
};