/****************************************************************************************** 
 *	Chili DirectX Framework Version 16.07.20											  *	
 *	Game.cpp																			  *
 *	Copyright 2016 PlanetChili.net <http://www.planetchili.net>							  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#include "MainWindow.h"
#include "Game.h"
#include "Camera.h"
#include "Mesh.h"
#include "Utils.h"
#include "Snake.h"
#include "AnimIterator.h"
#include <ctime>
#include <chrono>
typedef std::chrono::milliseconds milli;
Game::Game( MainWindow& wnd )
	:
	wnd( wnd ),
	gfx( wnd )
{
}

void Game::Go()
{
	gfx.BeginFrame();	
	UpdateModel();
	ComposeFrame();
	gfx.EndFrame();
}

std::vector<AnimIterator> animIterators;
Vec2f input(0, 0);
Snake snake(Vec2i(0,0));
Vec2i candy(4,4);

long long gettime() {
	return std::chrono::duration_cast<milli>(std::chrono::system_clock::now().time_since_epoch()) / std::chrono::milliseconds(1);
}

long long lastupdate = gettime();
long long timeBetweenUpdates = 500;
void Game::UpdateModel()
{
	if (!getInput(wnd).toInt().equals(Vec2i(0, 0))) {
		snake.vel = getInput(wnd).scale(2).toInt();
	}
	if (gettime() - lastupdate > timeBetweenUpdates) {
		lastupdate = gettime();
		

		snake.move();
		for (int i = 0; i < animIterators.size(); i++) {
			animIterators[i].at--;
		}
		int fieldsize = 10;
		snake.head.x = wrap(snake.head.x, -fieldsize, fieldsize);
		snake.head.y = wrap(snake.head.y, -fieldsize, fieldsize);


		if (snake.tryToEatCandy(candy)) {
			animIterators.push_back(AnimIterator(&snake));
		}
		if (snake.isColliding(snake.head)) {
			snake = Snake(Vec2i(0, 0));
		}
	}
	for (int i = 0; i < animIterators.size(); i++) {
		AnimIterator& animIt = animIterators[i];
		animIt.step();
		if (animIt.done) {
			animIterators.erase(animIterators.begin() + i);
		}
	}
}


void Game::ComposeFrame()
{
	
	std::vector<Mesh> bodyparts = snake.genBodyPartsMesh();
	Mesh head = snake.genHeadMesh();

	double t = sin(gettime() / (double)500);
	Mesh candyMesh = Mesh::cube();
	candyMesh.shader = new LitShader(Color(255, 0, 255));
	Matrix trs = Matrix::TRS(Vec3f(candy.x, candy.y, 20), Vec3f(t,t,t), Vec3f(1 + t, 1 + t, 1 + t));
	candyMesh.applyTransformation(trs);

	Camera camera;
	Vec2i sz(Graphics::ScreenWidth, Graphics::ScreenHeight);

	camera.draw(bodyparts, gfx);
	camera.draw(head, gfx);
	camera.draw(candyMesh, gfx);
}

