#pragma once
#include "Vector2.h"
#include "Vector3.h"

struct Matrix {

	float val[4][4];

	Matrix() {
		for (int rows = 0; rows < 4; rows++) {
			for (int cols = 0; cols < 4; cols++) {
				val[rows][cols] = 0;
			}
		}
	}

	Matrix mult(Matrix rhs) {
		Matrix matrix;
		for (int col = 0; col < 4; col++) {
			for (int row = 0; row < 4; row++) {
				float sum = 0;
				for (int i = 0; i < 4; i++) {
					sum += val[row][i] * rhs.val[i][col];
				}
				matrix.val[row][col] = sum;
			}
		}
		return matrix;
	}

	Vec3f mult(Vec3f& rhs) {
		Vec3f v(0,0,0);
		for (int rows = 0; rows < 3; rows++) {
			float sum = 0;
			for (int i = 0; i < 3; i++) {
				sum += val[rows][i] * rhs.get(i);
			}
			sum += val[rows][3];//all vectors are considered points
			v.set(rows, sum);
		}
		rhs = v;
		return rhs;
	}

	Matrix mult(float scalar) {
		for (int col = 0; col < 4; col++) {
			for (int row = 0; row < 4; row++) {
				val[row][col] *= scalar;
			}
		}
		return *this;
	}

	static Matrix translate(Vec3f translation) {
		Matrix matrix = Matrix::identity();
		for (int i = 0; i < 3; i++) {
			matrix.val[i][3] = translation.get(i);
		}
		return matrix;
	}

	static Matrix rot(Vec3f eulerAngles) {
		return Matrix::rotZ(eulerAngles.z).mult(Matrix::rotY(eulerAngles.y)).mult(Matrix::rotX(eulerAngles.x));
	}

	static Matrix rotX(float x) {
		Matrix m = Matrix::identity();
		float cosx = cos(x);
		float sinx = sin(x);
		m.val[1][1] = cosx;
		m.val[1][2] = -sinx;
		m.val[2][1] = sinx;
		m.val[2][2] = cosx;
		return m;
	}

	static Matrix rotY(float y) {
		Matrix m = Matrix::identity();
		float cosy = cos(y);
		float siny = sin(y);
		m.val[0][0] = cosy;
		m.val[0][2] = siny;
		m.val[2][0] = -siny;
		m.val[2][2] = cosy;
		return m;
	}

	static Matrix rotZ(float z) {
		Matrix m = Matrix::identity();
		float cosz = cos(z);
		float sinz = sin(z);
		m.val[0][0] = cosz;
		m.val[0][1] = -sinz;
		m.val[1][0] = sinz;
		m.val[1][1] = cosz;
		return m;
	}

	static Matrix scale(Vec3f scale) {
		Matrix matrix = Matrix::identity();
		for (int i = 0; i < 3; i++) {
			matrix.val[i][i] = scale.get(i);
		}
		return matrix;
	}

	static Matrix TRS(Vec3f pos, Vec3f eulerAngles, Vec3f scale) {
		return Matrix::translate(pos).mult(Matrix::rot(eulerAngles)).mult(Matrix::scale(scale));
	}

	static Matrix modelToWorld() {//TRS

	}

	static Matrix worldToView() {//camera inverse

	}

	static Matrix viewToProjection() {//and divide by w

	}

	static Matrix toScreenSpace(Vec2i sz) {//have to do yourself
		Matrix matrix;
		float halfScreenWidth = sz.x / (float)2;
		float halfScreenHeight = sz.y / (float)2;
		matrix.val[0][0] = halfScreenWidth;
		matrix.val[0][3] = halfScreenWidth;
		matrix.val[1][1] = -halfScreenHeight;
		matrix.val[1][3] = halfScreenHeight;

		return matrix;
	}

	static Matrix identity() {
		Matrix matrix;
		for (int i = 0; i < 4; i++) {
			matrix.val[i][i] = 1;
		}
		return matrix;
	}


};