#pragma once
#include "Vector2.h"
#include <vector>
#include "Mesh.h"

struct BodyPart {
	Vec2i pos;
	int age = 0;
	double rot = 0;

	BodyPart(Vec2i pos) :pos(pos) {

	}
};

struct Snake {
	Vec2i head;
	Vec2i vel = Vec2i(2, 0);
	int length = 3;
	std::vector<BodyPart> bodyParts;

	Snake(Vec2i pos) :head(pos) {
		
	}

	Mesh genHeadMesh() {
		Mesh headMesh = Mesh::cube();
		headMesh.shader = new LitShader(Color(0, 0, 255));
		Matrix trs = Matrix::TRS(Vec3f(head.x, head.y, 20), Vec3f(0, 0, 0), Vec3f(1, 1, 1));
		headMesh.applyTransformation(trs);
		return headMesh;
	}

	std::vector<Mesh> genBodyPartsMesh() {
		std::vector<Mesh> bodymeshes;
		for each(BodyPart bp in bodyParts) {
			Mesh bodyMesh = Mesh::cube();
			Matrix trs = Matrix::TRS(Vec3f(bp.pos.x, bp.pos.y, 20), Vec3f(0,0, 0), Vec3f(bp.rot / 6 + 1, bp.rot / 6 + 1, bp.rot / 6 + 1));
			bodyMesh.applyTransformation(trs);
			bodymeshes.push_back(bodyMesh);
		}
		return bodymeshes;
	}

	bool isColliding(Vec2i pos) {
		for (int i = 0; i < bodyParts.size(); i++) {
			if (pos.equals(bodyParts[i].pos)) {
				return true;
			}
		}
		return false;
	}

	void move() {
		bodyParts.push_back(BodyPart(head));
		head.add(vel);
		for (int i = 0; i < bodyParts.size(); i++) {
			bodyParts[i].age++;
			if (bodyParts[i].age > length) {
				bodyParts.erase(bodyParts.begin() + i);
			}
		}
	}

	bool tryToEatCandy(Vec2i& candy) {
		if (head.equals(candy)) {
			length++;
			Vec2i randspot = Vec2i(random(-4, 4), random(-4, 4)).scale(2);
			while (isColliding(randspot) && !randspot.equals(head)) {
				randspot = Vec2i(random(-4, 4), random(-4, 4)).scale(2);
			}
			candy = randspot;
			return true;
		}
		return false;
	}


};