#pragma once
template<typename T>
class Vector3
{
public:
	T x;
	T y;
	T z;

	Vector3() {};

	Vector3(T x, T y, T z) {
		this->x = x;
		this->y = y;
		this->z = z;
	};

	Vector3 normalise() {
		return scale(1 / length());
	}

	float length() {
		return sqrt(x * x + y * y + z * z);
	}

	float dot(Vector3& v) {
		return x * v.x + y * v.y + z * v.z;
	}

	Vector3 cross(Vector3& v) {
		return Vec3f(
			y * v.z - z * v.y,
			z * v.x - x * v.z,
			x * v.y - y * v.x
		);
	}

	Vector3 add(Vector3& v) {
		x += v.x;
		y += v.y;
		z += v.z;
		return *this;
	}

	Vector3 sub(Vector3& v) {
		x -= v.x;
		y -= v.y;
		z -= v.z;
		return *this;
	}

	Vector3 scale(float scalar) {
		x *= scalar;
		y *= scalar;
		z *= scalar;
		return *this;
	}

	Vector3 lerp(Vector3& v, float weight) {
		return c().add(v.c().sub((*this)).scale(weight));
	}

	T get(int i) {
		switch (i)
		{
		case 0: return x;
		case 1: return y;
		case 2: return z;
		}
		return x;
	}

	void set(int i, T val) {
		switch (i)
		{
		case 0: x = val;
		case 1: y = val;
		case 2: z = val;
		}
	}

	Vector3 c() {
		Vector3 c;
		c.x = x;
		c.y = y;
		c.z = z;
		return c;
	}

	Vector3<int> toInt() {
		Vec3i v;
		v.x = (int)x;
		v.y = (int)y;
		v.z = (int)z;
		return v;
	}

	Vector3<float> toFloat() {
		Vec3f v;
		v.x = (float)x;
		v.y = (float)y;
		v.z = (float)z;
		return v;
	}
};

typedef Vector3<float>	Vec3f;
typedef Vector3<int>	Vec3i;